@@ -6,10 +6,10 @@ use warnings;
 
 use AnyEvent;
 
-our $VERSION = '0.06_001';
+our $VERSION = '0.09_001';
 
 require XSLoader;
-XSLoader::load('AnyEvent::YACurl', $VERSION);
+XSLoader::load('AnyEvent::YACurl', '0.09_001');
 
 require constant;
 my %constants= %{_get_known_constants()};
